// 'inc_cf'
//

#include "x0_i0_partywide"

/////////////////////////////////////////////////////////////////////////////////////
//::                                CONSTANTS                                    :://
/////////////////////////////////////////////////////////////////////////////////////

const string sDES_CRAFT_DROP = "des_crft_drop";
const string sOPTIONS = "cf_options";

const int CF_OPTIONS_UseXPFix = 0;
const int CF_OPTIONS_UseCleavingFix = 1;
const int CF_OPTIONS_CheckAssociatesForCraftingRanks = 2;
const int CF_OPTIONS_CheckTotalCraftingRanks = 3;
const int CF_OPTIONS_CheckBaseCraftingRanksOnly = 4;
const int CF_OPTIONS_CheckCraftArmorSkill = 5;
const int CF_OPTIONS_CheckCraftWeaponSkill = 6;
const int CF_OPTIONS_CheckCraftAlchemySkill = 7;
const int CF_OPTIONS_CheckCraftTrapSkill = 8;
const int CF_OPTIONS_SingleSkillRankRequired = 9;
const int CF_OPTIONS_TotalSkillsRankRequired = 10;

int bUSE_XP_FIX = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_UseXPFix));
int bUSE_CLEAVING_FIX = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_UseCleavingFix));
int bCHECK_ASSOCIATES_FOR_CRAFTING_RANKS = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_CheckAssociatesForCraftingRanks));
int bCHECK_TOTAL_CRAFTING_RANKS = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_CheckTotalCraftingRanks));
int bCHECK_BASE_CRAFTING_RANKS_ONLY = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_CheckBaseCraftingRanksOnly));
int bCHECK_CRAFT_ARMOR_SKILL = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_CheckCraftArmorSkill));
int bCHECK_CRAFT_WEAPON_SKILL = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_CheckCraftWeaponSkill));
int bCHECK_CRAFT_ALCHEMY_SKILL = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_CheckCraftAlchemySkill));
int bCHECK_CRAFT_TRAP_SKILL = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_CheckCraftTrapSkill));
int iSINGLE_SKILL_RANK_REQUIRED = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_SingleSkillRankRequired));
int iTOTAL_SKILL_RANK_REQUIRED = StringToInt(Get2DAString(sOPTIONS,"Value",CF_OPTIONS_TotalSkillsRankRequired));

/////////////////////////////////////////////////////////////////////////////////////
//::                               PROTOTYPES                                    :://
/////////////////////////////////////////////////////////////////////////////////////

// Checks for stuff like required PC-faction skill-ranks in crafting and if so
// creates distillable loot.
void CreateDistillableLoot();

// Checks if party has required ranks in craft-skill(s).
int CheckCraftRanks(object oPC,
					int bAssociates = FALSE,
					int bTotal = FALSE,
					int bBaseOnly = FALSE);

// Checks if the player got the appropriate amount of XP for killing an enemy.
// If not, the player is awarded the additional XP based on the difference of
// what he already got and what he is actually supposed to get.
void GiveMissingXP();

// Checks if there're still enemies nearby and if it's still needed to play the animation
// for the cleave.
int CheckEnemy(object oPC);

// A wrapper function for the animation for the cleave. We use a wrapper function
// because PlayCustomAnimation() returns int while we need it to return void, for
// use inside DelayCommand().
void Cleave(object oPC);

// Plays the animation for the cleave when appropriate.
void PlayCleaveAnimation();

/////////////////////////////////////////////////////////////////////////////////////
//::                               DEFINITIONS                                   :://
/////////////////////////////////////////////////////////////////////////////////////

// Checks for stuff like required PC-faction skill-ranks in crafting and if so
// creates distillable loot.
// - called by OnDeath script
// - probably *not* a good idea to delay a call to this function
void CreateDistillableLoot()
{
	object oKiller = GetLastKiller();

	if (!GetIsObjectValid(oKiller)                              // always try to drop if killer is invalid
		|| (GetObjectType(oKiller) == OBJECT_TYPE_CREATURE      // else killer must be a creature
			&& (!GetIsObjectValid(GetFactionLeader(oKiller))    // because this checks for faction-leader (always do drop if not PC-faction)
				|| CheckCraftRanks(oKiller,
				                   bCHECK_ASSOCIATES_FOR_CRAFTING_RANKS,
				                   bCHECK_TOTAL_CRAFTING_RANKS,
				                   bCHECK_BASE_CRAFTING_RANKS_ONLY)))) // else this checks killer's party for required craft-ranks.
	{
		object oDead = OBJECT_SELF;

		int iAppearance = GetAppearanceType(oDead);
		//SendMessageToPC(GetFirstPC(FALSE), "appearance= " + IntToString(iAppearance));
		if (iAppearance < GetNum2DARows(sDES_CRAFT_DROP))
		{
			string sColType;
			string sColQty;
			string sColType2 = "";
			string sColQty2 = "";

			switch (GetObjectType(oDead))
			{
				case OBJECT_TYPE_CREATURE:
					sColType  = "Creature_Drop";
					sColQty   = "Creature_Num";
					sColType2 = "Creature_Drop_2";
					sColQty2  = "Creature_Num_2";
					break;

				case OBJECT_TYPE_DOOR:
					sColType = "Door_Drop";
					sColQty  = "Door_Num";
					break;

				case OBJECT_TYPE_PLACEABLE:
					sColType = "Placeable_Drop";
					sColQty  = "Placeable_Num";
					break;

				default: return;
			}

			int bCheck1 = FALSE;
			int bCheck2 = FALSE;
			string sResref  = Get2DAString(sDES_CRAFT_DROP, sColType,  iAppearance);
			string sResref2 = "";

			if (sResref  != "")
				bCheck1 = TRUE;

			if (sColType2 != "")
			{
				sResref2 = Get2DAString(sDES_CRAFT_DROP, sColType2, iAppearance);

				if (sResref2 != "")
					bCheck2 = TRUE;
			}

			if (bCheck1)
			{
				int hasInventory = GetHasInventory(oDead);

				int iQty  = StringToInt(Get2DAString(sDES_CRAFT_DROP, sColQty,  iAppearance));
				if (!iQty)  iQty  = 1;

				int i;

				if (hasInventory)
				{
					if (bCheck1)
					{
						for (i = 0; i != iQty; ++i)
							CreateItemOnObject(sResref, oDead);
					}

					if (bCheck2)
					{
						int iQty2 = StringToInt(Get2DAString(sDES_CRAFT_DROP, sColQty2, iAppearance));
						if (!iQty2) iQty2 = 1;

						for (i = 0; i != iQty2; ++i)
							CreateItemOnObject(sResref2, oDead);
					}

				}
				else
				{
					location lDead = GetLocation(oDead);

					if (bCheck1)
					{
						for (i = 0; i != iQty; ++i)
							CreateObject(OBJECT_TYPE_ITEM, sResref, lDead);
					}
				}
			}

		}
	}
}

// Checks if party has required ranks in craft-skill(s).
// - oPC         : PC-party member
// - bAssociates : true to check craft-ranks of associates also
// - bTotal      : true to compare against the total craft-ranks instead of
//                 separately
// - bBaseOnly   : true to use base skill ranks only, false to include skill
//                 modifiers
int CheckCraftRanks(object oPC,
					int bAssociates = FALSE,
					int bTotal = FALSE,
					int bBaseOnly = FALSE)
{
	object oFaction = GetFirstFactionMember(oPC, FALSE);
	while (GetIsObjectValid(oFaction))
	{
		if (GetAssociateType(oFaction) == ASSOCIATE_TYPE_NONE || bAssociates)
		{
			if (bTotal)
			{
				int nSkillRank = 0;

				if(bCHECK_CRAFT_ARMOR_SKILL)
				{
					nSkillRank += GetSkillRank(SKILL_CRAFT_ARMOR,   oFaction, bBaseOnly);
				}

				if(bCHECK_CRAFT_WEAPON_SKILL)
				{
					nSkillRank += GetSkillRank(SKILL_CRAFT_WEAPON,  oFaction, bBaseOnly);
				}

				if(bCHECK_CRAFT_ALCHEMY_SKILL)
				{
					nSkillRank += GetSkillRank(SKILL_CRAFT_ALCHEMY, oFaction, bBaseOnly);
				}

				if(bCHECK_CRAFT_TRAP_SKILL)
				{
					nSkillRank += GetSkillRank(SKILL_CRAFT_TRAP,    oFaction, bBaseOnly);
				}

				if(nSkillRank >= iTOTAL_SKILL_RANK_REQUIRED)
				{
					return TRUE;
				}
			}
			else
			{
				if (   (GetSkillRank(SKILL_CRAFT_ARMOR,   oFaction, bBaseOnly) >= iSINGLE_SKILL_RANK_REQUIRED && bCHECK_CRAFT_ARMOR_SKILL)
					|| (GetSkillRank(SKILL_CRAFT_WEAPON,  oFaction, bBaseOnly) >= iSINGLE_SKILL_RANK_REQUIRED && bCHECK_CRAFT_WEAPON_SKILL)
					|| (GetSkillRank(SKILL_CRAFT_ALCHEMY, oFaction, bBaseOnly) >= iSINGLE_SKILL_RANK_REQUIRED && bCHECK_CRAFT_ALCHEMY_SKILL
					|| (GetSkillRank(SKILL_CRAFT_TRAP,    oFaction, bBaseOnly) >= iSINGLE_SKILL_RANK_REQUIRED && bCHECK_CRAFT_TRAP_SKILL)))
				{
					return TRUE;
				}
			}
		}
		oFaction = GetNextFactionMember(oPC, FALSE);
	}
	return FALSE;
}

// Checks if the player got the appropriate amount of XP for killing an enemy.
// If not, the player is awarded the additional XP based on the difference of
// what he already got and what he is actually supposed to get.
// - called by OnDeath script
// - probably *not* a good idea to delay a call to this function
void GiveMissingXP()
{
	object oPC = GetFirstPC();
	object oKiller = GetLastKiller();
	int ChallengeRating = FloatToInt(GetChallengeRating(OBJECT_SELF));

	if(ChallengeRating == 0)
	{
		string ProperXP = Get2DAString("xptable", "C" + IntToString(GetHitDice(OBJECT_SELF)), GetHitDice(oKiller) - 1); //-1, because rows start with number 0
//		SendMessageToPC(GetFirstPC(FALSE), "last killer: " + GetTag(oKiller) + " " + GetName(oKiller)+" " +IntToString(GetFactionAverageLevel(oKiller))+ " " +ProperXP);

		string BrokenXP = Get2DAString("xptable", "C" + IntToString(ChallengeRating), GetHitDice(oKiller) - 1);
		int FinalXP = StringToInt(ProperXP) - StringToInt(BrokenXP);

		if(GetFactionEqual(oKiller, oPC) && FinalXP > 0)
		{
			// Always give XP to the player character, which will be then automatically added to all roster members
			// SendMessageToPC(GetFirstPC(FALSE), "Broken CR detected, awarding missing XP:");
			SendMessageToPC(GetFirstPC(FALSE), "Обнаружен некорректный Рейтинг сложности (CR) у существа. Получено недостающих очков опыта:");
			GiveXPToAll(oPC, FinalXP);
		}
	}

}

// Checks if there're still enemies nearby and if it's still needed to play the animation
// for the cleave.
// - oPC         : PC-party member
int CheckEnemy(object oPC)
{
	object oKiller = GetLastKiller();
	int nReturn = FALSE;
	float fRadius = StringToFloat(Get2DAString("appearance", "PREFATCKDIST", GetAppearanceType(oKiller)))*StringToFloat(Get2DAString("baseitems", "prefattackdist", GetBaseItemType(GetLastWeaponUsed(oKiller))));
	object oTarget = GetFirstObjectInShape(SHAPE_SPHERE, fRadius, GetLocation(oKiller), TRUE);

	while(GetIsObjectValid(oTarget))
	{
		if(oTarget != OBJECT_SELF && GetIsDead(oTarget) != TRUE && GetReputation(oKiller, oTarget) < 11)
		{
			nReturn = TRUE;
		}
		oTarget = GetNextObjectInShape(SHAPE_SPHERE, fRadius, GetLocation(oKiller), TRUE);
	}
	return nReturn;
}

// A wrapper function for an animation for the cleave. We use a wrapper function
// because PlayCustomAnimation() returns int while we need it to return void, for
// use inside DelayCommand().
// - oPC         : PC-party member
void Cleave(object oPC)
{
	PlayCustomAnimation(oPC, "*cleave", 0);
}

// Plays the animation for the cleave when appropriate.
void PlayCleaveAnimation()
{
	//Set the delay after which the animation is triggered
	float fDelay = 0.2;
	object oKiller = GetLastKiller();

	if(   GetHasFeat(FEAT_GREAT_CLEAVE, oKiller)
	   || GetHasFeat(FEAT_CLEAVE, oKiller))
	{
		if(CheckEnemy(oKiller) && GetLocalInt(oKiller, "cleave") != 1)
		{
			DelayCommand(fDelay, Cleave(oKiller));
			SetLocalInt(oKiller, "cleave", 1);
			DelayCommand(1.8, SetLocalInt(oKiller, "cleave", 0));
		}
	}
}